def org_user(pritunl_obj, org_name, user_name=None):
    def __get_org_by_name(orgs_obj, org_name):
        for org in orgs_obj:
            if org['name'] == org_name:
                return org
        return None

    def __get_user_by_name(users_obj, user_name):
        for user in users_obj:
            if user["name"] == user_name:
                return user
        return None

    org = __get_org_by_name(pritunl_obj.organization.get(), org_name)

    if user_name:
        user = __get_user_by_name(pritunl_obj.user.get(org_id=org['id']), user_name)
        return org, user
    return org



__version__ = "1.1.5"

import json
import logging as log
import sys
import base64
import hashlib
import hmac
import random
import requests
import time
import uuid
import os

requests.packages.urllib3.disable_warnings(
    requests.packages.urllib3.exceptions.InsecureRequestWarning)


class PritunlErr(Exception):
    def __init__(self, msg):
        log.error(msg)
        pass


class Pritunl:
    def __init__(self, url=None, token=None, secret=None):
        self.BASE_URL = self.clean_url(os.environ.get('PRITUNL_BASE_URL') if not url else url)
        self.API_TOKEN = os.environ.get('PRITUNL_API_TOKEN') if not token else token
        self.API_SECRET = os.environ.get('PRITUNL_API_SECRET') if not secret else secret

        # Sub classes
        self.server = self.ServerClass(self)
        self.organization = self.OrganizationClass(self)
        self.user = self.UserClass(self)
        self.key = self.KeyClass(self)

    class KeyClass:
        def __init__(self, root):
            self.r = None
            self.root = root

        def get(self, org_id=None, usr_id=None):
            try:
                if org_id and usr_id:
                    self.r = self.root.auth_request(method="GET", path="/key/{0}/{1}".format(org_id, usr_id))
                if self.r.status_code == 200:
                    return self.r
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
            except Exception:
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

    class UserClass:
        def __init__(self, root):
            self.r = None
            self.data_template = None
            self.headers = None
            self.root = root

        def get(self, org_id=None, usr_id=None):
            try:
                if org_id and not usr_id:
                    self.r = self.root.auth_request(method="GET", path="/user/{0}".format(org_id))
                elif org_id and usr_id:
                    self.r = self.root.auth_request(method="GET", path="/user/{0}/{1}".format(org_id, usr_id))
                if self.r.status_code == 200:
                    return self.r.json()
                else:
                    raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
            except Exception:
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

        def post(self, org_id=None, data=None):
            self.data_template = {
                'name': 'default_name',
                'email': None,
                'disabled': False,
            }
            self.data_template.update(data)
            try:
                self.headers = {'Content-Type': 'application/json'}
                self.r = self.root.auth_request(method="POST", path="/user/{0}".format(org_id),
                                                headers=self.headers,
                                                data=json.dumps(self.data_template))
                if self.r.status_code == 200:
                    return self.r.json()
                else:
                    raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
            except:
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

        def put(self, org_id=None, usr_id=None, data=None):
            try:
                self.headers = {'Content-Type': 'application/json'}
                self.r = self.root.auth_request(method="PUT", path="/user/{0}/{1}".format(org_id, usr_id),
                                                headers=self.headers,
                                                data=json.dumps(data))
                if self.r.status_code == 200:
                    return self.r.json()
                else:
                    raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
            except:
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

        def delete(self, org_id=None, usr_id=None):
            try:
                self.r = self.root.auth_request(method="DELETE", path="/user/{0}/{1}".format(org_id, usr_id))
                if self.r.status_code == 200:
                    return True
                else:
                    raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
            except Exception:
                return False
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

    class OrganizationClass:
        def __init__(self, root):
            self.r = None
            self.data_template = None
            self.headers = None
            self.root = root

        def get(self):
            try:
                self.r = self.root.auth_request(method="GET", path="/organization")
                if self.r.status_code == 200:
                    return self.r.json()
                else:
                    raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
            except Exception:
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

        def post(self, data=None):
            self.headers = {'Content-Type': 'application/json'}
            self.data_template = {
                'name': 'default_organization',
                'auth_api': False
            }
            self.data_template.update(data)
            try:
                self.r = self.root.auth_request(method="POST", path="/organization",
                                                headers=self.headers,
                                                data=json.dumps(self.data_template))
                if self.r.status_code == 200:
                    return self.r.json()
                else:
                    raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
            except Exception:
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

        def put(self, org_id=None, data=None):
            self.headers = {'Content-Type': 'application/json'}
            self.data_template = {
                'name': 'default_organization',
                'auth_api': False
            }
            self.data_template.update(data)
            try:
                self.r = self.root.auth_request(method="POST", path="/organization/{0}".format(org_id),
                                                headers=self.headers,
                                                data=json.dumps(self.data_template))
                if self.r.status_code == 200:
                    return self.r.json()
                else:
                    raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
            except Exception:
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

        def delete(self, org_id=None):
            try:
                self.r = self.root.auth_request(method="DELETE", path="/organization/{0}".format(org_id))
                if self.r.status_code == 200:
                    return True
                else:
                    raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
            except Exception:
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

    class ServerClass:
        def __init__(self, root):

            self.r = None
            self.root = root

        def get(self, srv_id=None, org=None, out=None):
            try:
                if srv_id and not org and not out:
                    self.r = self.root.auth_request(method="GET", path="/server/{0}".format(srv_id))
                elif srv_id and org and not out:
                    self.r = self.root.auth_request(method="GET", path="/server/{0}/organization".format(srv_id))
                elif srv_id and out and not org:
                    self.r = self.root.auth_request(method="GET", path="/server/{0}/output".format(srv_id))
                else:
                    self.r = self.root.auth_request(method="GET", path="/server")
                if self.r.status_code == 200:
                    return self.r.json()
                else:
                    raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
            except Exception:
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

        def delete(self, srv_id=None, org_id=None, out=None):
            try:
                if srv_id and not out and not org_id:
                    self.r = self.root.auth_request(method="DELETE", path="/server/{0}".format(srv_id))
                if srv_id and out and not org_id:
                    self.r = self.root.auth_request(method="DELETE", path="/server/{0}/output".format(srv_id))
                if srv_id and org_id and not out:
                    self.r = self.root.auth_request(method="DELETE",
                                                    path="/server/{0}/organization/{1}".format(srv_id, org_id))
                if self.r.status_code == 200:
                    return True
                else:
                    raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
            except Exception:
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

        def put(self, srv_id=None, operation=None, org_id=None, data=None):
            self.header = {'Content-Type': 'application/json'}
            try:
                if operation and not data and srv_id and not org_id:
                    self.r = self.root.auth_request(method="PUT", path="/server/{0}/operation/{1}".format(srv_id,operation))
                if srv_id and data and not operation and not org_id:
                    self.r = self.root.auth_request(method="PUT",
                                                    path="/server/{0}".format(srv_id),
                                                    headers=self.header,
                                                    data=json.dumps(data))
                if srv_id and org_id and not data and not operation:
                    self.r = self.root.auth_request(method="PUT",
                                                    path="/server/{0}/organization/{1}".format(srv_id, org_id)
                                                    )
                if self.r.status_code == 200:
                    return self.r.json()
                else:
                    raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
            except:
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

        def post(self, data=None):
            self.data_template = {
                'name': 'default_server',
                'network': '10.{0}.{1}.0/24'.format(random.randrange(1,254), random.randrange(1,254)),
                'groups': [],
                'network_mode': 'tunnel',
                'network_start': None,
                'network_end': None,
                'restrict_routes': True,
                'ipv6': False,
                'ipv6_firewall': True,
                'bind_address': None,
                'port': random.randrange(1025,20000),
                'protocol': 'tcp',
                'dh_param_bits': 1536,
                'multi_device': False,
                'dns_ervers': ['8.8.8.8'],
                'search_domain': '',
                'otp_auth': False,
                'cipher': 'aes128',
                'hash': 'sha1',
                'jumbo_frames': False,
                'lzo_compression': False,
                'inter_client': True,
                'ping_interval': 10,
                'ping_timeout': 60,
                'link_ping_interval': 1,
                'link_ping_timeout': 5,
                'onc_hostname': None,
                'allowed_devices': None,
                'max_clients': 10,
                'replica_count': 1,
                'vxlan': True,
                'dns_mapping': False,
                'debug': False,
                'policy': None
            }
            self.data_template.update(data)
            try:
                self.header = {'Content-Type': 'application/json'}
                self.r = self.root.auth_request(method="POST", path="/server",
                                                headers=self.header,
                                                data=json.dumps(self.data_template))
                if self.r.status_code == 200:
                    return self.r.json()
                else:
                    raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
            except:
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

    def test(self):
        pass

    def ping(self):
        try:
            self.r = self.auth_request(method="GET", path="/ping")
            if self.r.status_code == 200:
                return True
            else:
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
        except Exception:
            raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

    def check(self):
        try:
            self.r = self.auth_request(method="GET", path="/check")
            if self.r.status_code == 200:
                return True
            else:
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
        except Exception:
            raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

    def setting(self):
        try:
            self.r = self.auth_request(method="GET", path="/settings")
            if self.r.status_code == 200:
                return self.r.json()
            else:
                raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))
        except Exception:
            raise PritunlErr("{0}:{1}".format(sys._getframe().f_code.co_name, self.root.BASE_URL))

    def lastResponse(self):
        return self.r.json()

    def auth_request(self, method, path, headers=None, data=None):
        auth_timestamp = str(int(time.time()))
        auth_nonce = uuid.uuid4().hex
        auth_string = '&'.join([self.API_TOKEN, auth_timestamp, auth_nonce,
                                method.upper(), path])

        hmacv = hmac.new(str.encode(self.API_SECRET), auth_string.encode('utf-8'), hashlib.sha256).digest()

        auth_signature = base64.b64encode(hmacv)
        auth_headers = {
            'Auth-Token': self.API_TOKEN,
            'Auth-Timestamp': auth_timestamp,
            'Auth-Nonce': auth_nonce,
            'Auth-Signature': auth_signature
        }

        if headers:
            auth_headers.update(headers)
        return getattr(requests, method.lower())(
            self.BASE_URL + path,
            verify=False,
            headers=auth_headers,
            data=data,
            timeout=30
        )

    def clean_url(self, url):
        return url.rstrip('/')



import click

# Pritunl
from .commands import users

@click.group()
def run():
    pass

# Get User
@run.command()
@click.option('--org-name')
@click.option('--user-name')
@click.option('--get-profile-key-only', is_flag=True)
def get_user(**kwargs):
    users.get_user(**kwargs)

# Create User
@run.command()
@click.option('--org-name')
@click.option('--user-name')
@click.option('--user-email')
@click.option('--pin')
@click.option('--yubikey-id')
@click.option('--from-csv-file', type=click.Path(exists=True))
def create_user(**kwargs):
    users.create_user(**kwargs)

# Update User
@run.command()
@click.option('--org-name')
@click.option('--user-name')
@click.option('--disable/--enable', default=False)
def update_user(**kwargs):
    users.update_user(**kwargs)

# Delete User
@run.command()
@click.option('--org-name')
@click.option('--user-name')
def delete_user(**kwargs):
    users.delete_user(**kwargs)

if __name__ == '__main__':
    run()



import csv
import json
from urllib.parse import urlparse

from pritunl_api import Pritunl
pritunl = Pritunl()

from pritunl_api.utils.query import org_user
from pritunl_api.utils.genkey import profile_key

import click

from rich import print_json
from rich.console import Console
console = Console(width=160)


def get_user(**kwargs):
    org, user = org_user(pritunl_obj=pritunl, org_name=kwargs['org_name'], user_name=kwargs['user_name'])
    key_uri_url, key_view_url = profile_key(pritunl_obj=pritunl, org_id=org['id'], usr_id=user['id'])

    if kwargs['get_profile_key_only']:
        console.print(
            f"USER KEY INFORMATION FOR `{user['name']}` FROM `{org['name']}` ORGANIZATION",
            f"TEMPORARY PROFILE KEY (Expires after 24 hours)",
            style="green bold", sep='\n'
        )
        console.print(
            f"PROFILE URI (PRITUNNL CLIENT IMPORT PROFILE): '{key_uri_url}'",
            f"PROFILE URL (WEB VIEW PROFILE): '{key_view_url}'",
            style="blue", sep='\n', end='\n \n', new_line_start=True
        )
    else:
        console.print(
            f"USER INFORMATION FOR `{user['name']}` FROM `{org['name']}` ORGANIZATION",
            style="green bold", end='\n \n'
        )
        print_json(json.dumps(user))


def create_user(**kwargs):
    def __create_user(org_id, user_name, user_email):
        user_data = {
            'name': user_name,
            'email': user_email
        }

        if kwargs['pin']:
            user_data["pin"] = kwargs['pin']

        if kwargs['yubikey_id']:
            user_data["auth_type"] = "yubico"
            user_data["yubico_id"] = kwargs['yubikey_id'][:12]

        create_user = pritunl.user.post(org_id=org_id, data=user_data)
        for user in create_user:
            key_uri_url, key_view_url = profile_key(pritunl_obj=pritunl, org_id=org['id'], usr_id=user['id'])
            console.print(
                f"USER `{user['name']}` WITH AN EMAIL `{user['email']}` FOR `{user['organization_name']}` ORGANIZATION IS SUCCESSFULLY CREATED!",
                f"TEMPORARY PROFILE KEY (Expires after 24 hours)",
                style="green bold", sep='\n', new_line_start=True
            )

            console.print(
                f"PROFILE URI (PRITUNNL CLIENT IMPORT PROFILE): '{key_uri_url}'",
                f"PROFILE URL (WEB VIEW PROFILE): '{key_view_url}'",
                style="blue", sep='\n', end='\n \n', new_line_start=True
            )

    if kwargs['org_name'] and kwargs['user_name'] and kwargs['user_email'] and not kwargs['from_csv_file']:
        org, user = org_user(pritunl_obj=pritunl, org_name=kwargs['org_name'], user_name=kwargs['user_name'])
        if not user:
            __create_user(org_id=org['id'], user_name=kwargs['user_name'], user_email=kwargs['user_email'])
        else:
            console.print(
                f"User `{kwargs['user_name']}` already exist for organization `{kwargs['org_name']}`, escaping user creation!",
                style="red bold", new_line_start=True, end='\n \n'
            )

    elif kwargs['from_csv_file'] and not kwargs['org_name'] and not kwargs['user_name'] and not kwargs['user_email']:
        csv_list = []
        with open(kwargs['from_csv_file']) as csvfile:
            reader = csv.DictReader(csvfile, skipinitialspace=True)
            for row in reader:
                csv_list.append(row)

        for row in csv_list:
            org, user = org_user(pritunl_obj=pritunl, org_name=row['Organization'], user_name=row['Username'])
            if not user:
                __create_user(org_id=org['id'], user_name=row['Username'], user_email=row['Email'])
            else:
                console.print(
                    f"User `{row['Username']}` already exist for organization `{row['Organization']}`, escaping user creation!",
                    style="red bold"
                )
            console.rule()

    else:
        if not kwargs['org_name'] and not kwargs['user_name'] and not kwargs['user_email'] and not kwargs['from_csv_file']:
            raise click.UsageError('Error: You entered with empty options.')
        else:
            raise click.UsageError('Error: You entered an invalid combination of options.')



def update_user(**kwargs):
    org, user = org_user(pritunl_obj=pritunl, org_name=kwargs['org_name'], user_name=kwargs['user_name'])
    user_data = {
        'name': user['name'],
        'email': user['email'],
        'disabled': False,
    }

    if kwargs['disable']:
        user_data.update({'disabled': True})

    response = pritunl.user.put(org_id=org['id'], usr_id=user['id'], data=user_data)
    if response:
        console.print(
            f"USER `{user['name']}` FROM `{org['name']}` ORGANIZATION WAS SUCCESSFULLY `{'DISABLED' if response['disabled']==True else 'ENABLED'}`",
            style="green bold", sep='\n', new_line_start=True
        )


def delete_user(**kwargs):
    org, user = org_user(pritunl_obj=pritunl, org_name=kwargs['org_name'], user_name=kwargs['user_name'])
    response = pritunl.user.delete(org_id=org['id'], usr_id=user['id'])

    if response:
        console.print(
            f"USER `{user['name']}` FROM `{org['name']}` ORGANIZATION WAS SUCCESSFULLY DELETED",
            style="green bold", sep='\n', new_line_start=True
        )



from urllib.parse import urlparse

def profile_key(pritunl_obj, org_id, usr_id):
    key = pritunl_obj.key.get(org_id=org_id, usr_id=usr_id)
    if key:
        key_uri_url = urlparse(pritunl_obj.BASE_URL)._replace(scheme='pritunl').geturl() + key.json()['uri_url']
        key_view_url =  pritunl_obj.BASE_URL + key.json()['view_url']
        return key_uri_url, key_view_url
    else:
        return None, None
